@@ -1,13 +1,40 @@
 #pragma once
 
 static const char* szVEROROUTE_VERSION =
-
-"2.28";		// Bug fix: Code for X-thermals not properly handling isolated ground pad on 2-layer board.
+"2.36";		// Improvement: Automatically force text-box text to black or white when in Mono mode.
+//"2.35";	// New feature: Added View menu item to toggle between Inverse/Normal Mono mode.
+			// New feature: Added Ground Fill Perimeter parameter to PCB parameters.
+			//              Made the Board Edge Margin (PCB parameter) apply to all view types, and account for it when drawing to screen.
+			// Bug fix: Writing to PDF was not centering the page correctly.
+			// Bug fix: Metric Gerber files were not as compressed as they could be.
+			// Improvement: If closing via X-button or windows toolbar, ask for confirmation if layout has been modified.
+//"2.34";	// Bug fix: Component editor was not storing pin info correctly, so pins of placed part not protected by regular track painting.
+			// Improvement: Added "Yes To All" option for overwriting library parts when adding to library from VRT/layout.
+//"2.33";	// Bug fix: Label offsets were not persisted by the Component Editor.
+			// Bug fix: If not forcing X-thermals, the code for producing additional thermal relief tags was missing extra checks for tags in the diagonal direction.
+			// Bug fix: File -> Open (merge into current) could show the newly loaded board with PCB style shapes while in Color mode.
+			// Bug fix: Autocrop and Margin controls were not immediately updating the board size info in the status bar.
+			// Bug fix: The Generic Parts list in the Parts Library should not mention the number of default pins (e.g. should say DIP not DIP8).
+			// Bug fix: Vias within the ground fill were not rendered correctly, especially if forcing X-thermals.
+			// Bug fix: Code for placing board holes (i.e. unusable locations) was only checking that the bottom layer was not painted instead of all layers.
+			// Bug fix: Could get duplicate target points in the routing list.
+			// Improvement: Allow non-custom library parts (e.g. T092) to be overwritten if the Import string has not changed but the Prefix string has.
+			// Improvement: Allow editing of a library part without placing it, by selecting it in the Parts Library then switching to Component Editor mode.
+			// New feature: Added support for some SOICs (8,14,16,20,24,28 pin).  See Tutorial 26.
+//"2.32";	// Bug fix: When adding a new library part, should allow overwrite of an existing library part
+			//          with matching Type string and matching (non-empty) Import string, but different Value string.
+			// Added veroroute.appdata.xml metainfo to installation.
+//"2.31";	// New feature: Part Aliases dialog for easier import of netlists with unknown footprint/package types.
+			// Improvement: Allow library parts to be overwritten.
+//"2.30";	// Gerber Generator version 1.0.  Write drill file using XNC format as defined in Ucamco specification.
+			// Draw pads as squares in PCB mode.  Increased size of LED symbol.
+//"2.29";	// Increased routing algorithm speed by 30%.  Replaced "Templates" terminology with "Parts Library" and updated tutorials.
+//"2.28";	// Bug fix: Code for X-thermals not properly handling isolated ground pad on 2-layer board.
 //"2.27";	// Bug fix: Bug in component manager leads to incorrect warning about circuit not being saved (even if it has been).
 			// Disabled use of double-click for rotating parts.
 //"2.26";	// Removed infrequently used icons from the toolbar.  Added tutorial info on text boxes.
 //"2.25";	// Bug fix: Major bug.  Double-clicking on a user-defined template to add the part to the layout
-			//			can give the wrong footprint unless the part was created using component editor.
+			//          can give the wrong footprint unless the part was created using component editor.
 			// Bug fix: Some actions in the Key/Mouse Actions Dialog were removed in V2.20 but still listed.
 //"2.24";	// Bug fix: Broken Nets list was not updating the main view if user scrolled to a list item instead of clicking on it.
 //"2.23";	// New feature: Option to force thermal reliefs to use an X-shaped pattern of tags.
@@ -332,7 +359,8 @@ static const char* szVEROROUTE_VERSION =
 //"1.24";	// Added IC Pin Labels dialog.
 //"1.23";	// Fixed memory leak on shutdown.
 //			// Updated Tutorial 19 with info on creating a registry key for MS Windows.
-//			// Added more JFETs to the symbol libraries.//"1.22";	// Added command line options to specify VRT file and path to VeroRoute home directory.
+//			// Added more JFETs to the symbol libraries.
+//"1.22";	// Added command line options to specify VRT file and path to VeroRoute home directory.
 //			// Support drag and drop of VRT files from file explorer.
 //			// Allow multiple VeroRoute instances to run at the same time.
 //			// Improved error reporting during netlist import.
