@@ -19,7 +19,6 @@
 
 #pragma once
 
-#include "VeroRouteAndroid.h"
 #include <QtGui>
 #include <QApplication>
 #include <QColorDialog>
@@ -37,8 +36,8 @@
 #include "GWriter.h"
 #include "myscrollarea.h"
 
-#ifndef VEROROUTE_ANDROID
-#define USE_PIXMAP_CACHE
+#ifdef _GRID_DEBUG
+enum DEBUGMODE { DEBUGMODE_OFF = 0 , DEBUGMODE_SOICINFO, DEBUGMODE_ROUTEID, DEBUGMODE_NODEID, DEBUGMODE_PININDEX, DEBUGMODE_LAYERINFO, DEBUGMODE_END };
 #endif
 
 namespace Ui { class MainWindow; }
@@ -52,6 +51,7 @@ class InfoDialog;
 class CompDialog;
 class TextDialog;
 class BomDialog;
+class AliasDialog;
 class PinDialog;
 class PadOffsetDialog;
 class FindDialog;
@@ -74,6 +74,7 @@ class MainWindow : public QMainWindow
 	friend class CompDialog;
 	friend class TextDialog;
 	friend class BomDialog;
+	friend class AliasDialog;
 	friend class PinDialog;
 	friend class PadOffsetDialog;
 	friend class FindDialog;
@@ -104,7 +105,7 @@ public:
 	// Helper for mouse
 	void GetPixMapXY(const QPoint& currentPoint, int& pixmapX, int& pixmapY) const;
 	bool GetRowCol(const QPoint& currentPoint, int& row, int& col, double& deltaRow, double& deltaCol) const;
-	bool GetRowCol(const QPoint& currentPoint, const int rows, const int cols, int& row, int& col, double& deltaRow, double& deltaCol) const;
+	bool GetRowCol(const QPoint& currentPoint, int rows, int cols, int& row, int& col, double& deltaRow, double& deltaCol) const;
 	bool HaveZeroDeltaRowCol(int& deltaRow, int& deltaCol);
 
 	// Helpers for rendering
@@ -142,14 +143,15 @@ public:
 
 	const QString&	GetFileName() const { return m_fileName; }
 
-	bool GetHaveFloatingPin(int& iFloatingNodeId);
-	void MousePressEvent(const QPoint& pos, const bool& bLeftClick = true, const bool& bRightClick = true);
+	bool GetHaveFloatingPinTH(int& iFloatingNodeId);
+	void MousePressEvent(const QPoint& pos, bool bLeftClick = true, bool bRightClick = true);
 	void MouseDoubleClickEvent(const QPoint& pos);
 	void MouseMoveEvent(const QPoint& pos);
 	void MouseReleaseEvent(const QPoint& pos);
 protected:
 	void paintEvent(QPaintEvent* event);
 	void wheelEvent(QWheelEvent* event);
+	void closeEvent(QCloseEvent* event);
 	void keyPressEvent(QKeyEvent* event);
 	void keyReleaseEvent(QKeyEvent* event);
 #ifndef VEROROUTE_ANDROID
@@ -169,11 +171,13 @@ public slots:
 	void Merge();
 	void Save();
 	void SaveAs();
-	void ImportTango();
-	void ImportOrcad();
+	void ReImport();
+	void Import(bool bTango);
+	void ImportTango() { Import(true); }	// true  ==> Tango
+	void ImportOrcad() { Import(false); }	// false ==> Orcad
 	void WritePDF();
 	void WritePNG();
-	void WriteGerber(const bool& bTwoLayerGerber, const bool& bMetric);
+	void WriteGerber(bool bTwoLayerGerber, bool bMetric);
 	void WriteGerber1in()	{ WriteGerber(false, false); }
 	void WriteGerber1mm()	{ WriteGerber(false, true); }
 	void WriteGerber2in()	{ WriteGerber(true, false); }
@@ -190,6 +194,7 @@ public slots:
 	void ToggleFlipV();
 	void TogglePinLabels();
 	void ToggleFlyWires();
+	void ToggleInverseMono();
 	void ToggleRuler();
 	// Toolbar items
 	void VeroV();
@@ -253,6 +258,14 @@ public slots:
 	void AddTrimFlatWide()		{ AddPart(COMP::TRIM_FLAT_WIDE); }
 	void AddSIP()				{ AddPart(COMP::SIP); }
 	void AddDIP()				{ AddPart(COMP::DIP); }
+	void AddSOIC8()				{ AddPart(COMP::SOIC8); }
+	void AddSOIC14()			{ AddPart(COMP::SOIC14); }
+	void AddSOIC16()			{ AddPart(COMP::SOIC16); }
+	void AddSOIC14W()			{ AddPart(COMP::SOIC14W); }
+	void AddSOIC16W()			{ AddPart(COMP::SOIC16W); }	
+	void AddSOIC20W()			{ AddPart(COMP::SOIC20W); }
+	void AddSOIC24W()			{ AddPart(COMP::SOIC24W); }
+	void AddSOIC28W()			{ AddPart(COMP::SOIC28W); }	
 	void AddStrip100()			{ AddPart(COMP::STRIP_100); }
 	void AddBlock100()			{ AddPart(COMP::BLOCK_100); }
 	void AddBlock200()			{ AddPart(COMP::BLOCK_200); }
@@ -292,6 +305,9 @@ public slots:
 	void HidePinDialog();
 	void ShowBomDialog();
 	void HideBomDialog();
+	void ShowAliasDialog();
+	void ShowAliasDialog_NoFile();
+	void HideAliasDialog();
 	void ShowWireDialog();
 	void HideWireDialog();
 	void ShowFindDialog();
@@ -341,14 +357,15 @@ public slots:
 	void SetFlipV(bool b);
 	void SetShowPinLabels(bool b);
 	void SetShowFlyWires(bool b);
+	void SetInverseMono(bool b);
 	void SetFill(bool b);
 	// Part controls
 	void SetCompName(const QString& str);
 	void SetCompValue(const QString& str);
 	void SetCompType(const QString& str);
-	void SetCompCustomFlag(const bool& b);
-	void SetCompPadWidth(const int& i);
-	void SetCompHoleWidth(const int& i);
+	void SetCompCustomFlag(bool b);
+	void SetCompPadWidth(int i);
+	void SetCompHoleWidth(int i);
 	void CompRotateCCW()	{ CompRotate(false); }
 	void CompRotateCW()		{ CompRotate(true); }
 	void CompGrow()			{ CompStretch(true); }
@@ -397,6 +414,7 @@ public slots:
 	void SetMaskWidth(int i);
 	void SetSilkWidth(int i);
 	void SetEdgeWidth(int i);
+	void SetFillWidth(int i);
 	void SetViaPadWidth(int i);
 	void SetViaHoleWidth(int i);
 	void SetTextSizeComp(int i);
@@ -483,40 +501,37 @@ private:
 	void UpdateCompDialog();
 	void EnableCompDialogControls();
 	void UpdateBOM();
+	void UpdateAliasDialog();
 	void UpdateTemplatesDialog();
 	void UpdateTextDialog(bool bFull = false);
 
-	void DestroyPixmapCache();
-#ifdef USE_PIXMAP_CACHE
-	void CreatePixmapCache(const GuiControl& guiCtrl, ColorManager& colorManager);
-	void PaintDiag(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pCorner, bool bLT);
-#endif
 	void PaintViaGrey(const GuiControl& guiCtrl, QPainter& painter, const QPointF& pC);
-	void PaintPadGrey(const GuiControl& guiCtrl, QPainter& painter, QPen& pen, const QPointF& pC, const int& iPadWidthMIL = 0);
-	void PaintVia(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, const bool& bGap = false);	// Helper
-	void PaintPad(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, const int& iPadWidthMIL = 0, const int& iHoleWidth_MIL = 0, const bool& bGap = false);	// Helper
+	void PaintPadGrey(const GuiControl& guiCtrl, QPainter& painter, QPen& pen, const QPointF& pC, int iPadWidthMIL = 0);
+	void PaintSOIC(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, size_t pinIndex, const Component* pComp, bool bIsGnd, bool bGap = false);
+	void PaintVia(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, bool bGap = false);	// Helper
+	void PaintPad(const GuiControl& guiCtrl, QPainter& painter,  const QColor& color, const QPointF& pC, int iPadWidthMIL = 0, int iHoleWidth_MIL = 0, bool bGap = false);	// Helper
 	void PaintBlob(const GuiControl& guiCtrl, QPainter& painter, const QColor& color, const QPointF& pC, const QPointF& pCoffset,
-				   const int& iPadWidthMIL, const int& iPerimeterCode, const int& iTagCode,
-				   const bool bHavePad, const bool bIsGnd, const bool bGap = false);	// Helper
+				   int iPadWidthMIL, int iPerimeterCode, int iTagCode,
+				   bool bHavePad, bool bHaveSoic, bool bIsGnd, bool bGap = false);	// Helper
 	void PaintBoard();
 	void PaintCompDefiner();
-	void HandleRouting(const bool bSingleRoute = false);
+	void HandleRouting(bool bSingleRoute = false);
 	void RepaintWithListNodes(bool bNow = false);
 	void RepaintWithRouting(bool bNow = false);
 	void RepaintSkipRouting(bool bNow = false);
 	void ShowCurrentRectSize();	// Show current rect size in status bar
-	int	 GetCurrentLayer() const			{ return m_board.GetCurrentLayer(); }
-	int	 GetCurrentNodeId() const			{ return m_board.GetCurrentNodeId(); }
-	int  GetCurrentCompId() const			{ return m_board.GetCurrentCompId(); }
-	int  GetCurrentTextId() const			{ return m_board.GetCurrentTextId(); }
-	int  GetCurrentPinId() const			{ return m_board.GetCurrentPinId(); }
-	int  GetCurrentShapeId() const			{ return m_board.GetCurrentShapeId(); }
-	void SetCurrentNodeId(const int& i)		{ m_board.SetCurrentNodeId(i);	ListNodes(false); }
-	void SetCurrentCompId(const int& i)		{ m_board.SetCurrentCompId(i);	UpdateControls(); }
-	void SetCurrentTextId(const int& i)		{ m_board.SetCurrentTextId(i);	UpdateControls(); }
-	void SetCurrentPinId(const int& i)		{ m_board.SetCurrentPinId(i);	UpdateCompDialog(); }
-	void SetCurrentShapeId(const int& i)	{ m_board.SetCurrentShapeId(i);	UpdateCompDialog(); UpdateControls(); }
-	TextRect& GetCurrentTextRect()			{ return m_board.GetTextMgr().GetTextRectById( GetCurrentTextId() ); }
+	int	 GetCurrentLayer() const	{ return m_board.GetCurrentLayer(); }
+	int	 GetCurrentNodeId() const	{ return m_board.GetCurrentNodeId(); }
+	int  GetCurrentCompId() const	{ return m_board.GetCurrentCompId(); }
+	int  GetCurrentTextId() const	{ return m_board.GetCurrentTextId(); }
+	int  GetCurrentPinId() const	{ return m_board.GetCurrentPinId(); }
+	int  GetCurrentShapeId() const	{ return m_board.GetCurrentShapeId(); }
+	void SetCurrentNodeId(int i)	{ m_board.SetCurrentNodeId(i);	ListNodes(false); }
+	void SetCurrentCompId(int i)	{ m_board.SetCurrentCompId(i);	UpdateControls(); }
+	void SetCurrentTextId(int i)	{ m_board.SetCurrentTextId(i);	UpdateControls(); }
+	void SetCurrentPinId(int i)		{ m_board.SetCurrentPinId(i);	UpdateCompDialog(); }
+	void SetCurrentShapeId(int i)	{ m_board.SetCurrentShapeId(i);	UpdateCompDialog(); UpdateControls(); }
+	TextRect& GetCurrentTextRect()	{ return m_board.GetTextMgr().GetTextRectById( GetCurrentTextId() ); }
 	// Helpers for slots
 	void ZoomHelper(int delta);
 	void AddPart(COMP eType)
@@ -535,6 +550,9 @@ private:
 		SetCurrentCompId(compId);
 
 		UpdateHistory("add part"); UpdateControls(); UpdateBOM(); RepaintSkipRouting();
+
+		if ( CompTypes::GetIsSOIC(eType) && m_board.GetLyrs() == 1 && m_board.GetCompMgr().GetNumSOIC() == 1 )	// If it's the only SOIC on a 1-layer board
+			QMessageBox::information(this, tr("Information"), tr("SOICs can only be placed on a 2-layer board.  Add a second layer by doing 'Layers -> Add Top Layer'."));
 	}
 	void AddFromTemplate(bool bGeneric, const Component& compTemp)
 	{
@@ -555,11 +573,11 @@ private:
 
 		UpdateHistory("add part"); UpdateControls(); UpdateBOM(); RepaintSkipRouting();
 	}
-	void CompRotate(const bool& bCW);
-	void CompStretch(const bool& bGrow);
-	void CompStretchWidth(const bool& bGrow);
-	void CompTextMove(const int& deltaRow, const int& deltaCol);
-	void PadMove(const int& deltaRowMil, const int& deltaColMil);
+	void CompRotate(bool bCW);
+	void CompStretch(bool bGrow);
+	void CompStretchWidth(bool bGrow);
+	void CompTextMove(int deltaRow, int deltaCol);
+	void PadMove(int deltaRowMil, int deltaColMil);
 	void UpdatePadInfo();
 
 	// Helpers
@@ -573,9 +591,9 @@ private:
 	bool GetIsModified() const;
 	bool GetMatchesVrtFile(const std::string& fileName) const;
 	void ResetHistory(const std::string& str);
-	void UpdateHistory(const std::string& str, const int objId = -1);
+	void UpdateHistory(const std::string& str, int objId = -1);
 	void UpdateUndoRedoControls();
-	void SetMouseActionString(const std::string& str, const int objId = -1);
+	void SetMouseActionString(const std::string& str, int objId = -1);
 
 	// Helper to auto-append suffix when writing a file
 	QString GetSaveFileName(const QString& caption, const QString& nameFilter, const QString& defaultSuffix);
@@ -622,6 +640,7 @@ private:
 	HotkeysDialog*			m_hotkeysDlg		= nullptr;
 	TextDialog*				m_textDlg			= nullptr;
 	BomDialog*				m_bomDlg			= nullptr;
+	AliasDialog*			m_aliasDlg			= nullptr;
 	PadOffsetDialog*		m_padOffsetDlg		= nullptr;
 	FindDialog*				m_findDlg			= nullptr;
 
@@ -637,19 +656,6 @@ private:
 	std::string				m_localDataPathStr;	// The path to the "history" and "templates" folders
 	std::string				m_tutorialsPathStr;	// The path to the "tutorials" folder and "veroroute.png"
 
-	// Cached pixmaps containing pre-colored pads and blobs.
-#ifdef USE_PIXMAP_CACHE
-	QPixmap**	m_ppPixmapPad		= nullptr;	// A pad in the host element
-	QPixmap**	m_ppPixmapVia		= nullptr;	// A via in the host element
-	QPixmap**	m_ppPixmapDiag		= nullptr;	// For filling small diagonal gaps not covered by blob pixmaps
-	QPixmap**	m_ppPixmapBlob		= nullptr;	// A composite shape with all the host element connections
-	QPixmap*	m_pPixmapDiagLT		= nullptr;	// Used instead of m_ppPixmapDiag for custom colors
-	QPixmap*	m_pPixmapDiagRT		= nullptr;	// Used instead of m_ppPixmapDiag for custom colors
-	int			m_radPixmapPad		= 0;		// Half pixmap width ...
-	int			m_radPixmapVia		= 0;		// ...
-	int			m_radPixmapDiag		= 0;		// ...
-	int			m_radPixmapBlob		= 0;		// ...
-#endif
 	QPoint		m_mousePos;
 	QPoint		m_clickedPos;
 	bool		m_bRepaint			= false;	// Flag to make paintEvent() do something useful
@@ -666,6 +672,7 @@ private:
 	bool		m_bTwoLayerGerber	= false;	// true ==> 2-layer Gerber output instead of 1-layer
 	bool		m_bHistoryDir		= false;	// true ==> have "history" folder
 	bool		m_bTemplatesDir		= false;	// true ==> have "templates" folder
+	bool		m_bAliasesDir		= false;	// true ==> have "aliases" folder
 	bool		m_bRuler			= false;
 	bool		m_bModifyRulerA		= false;
 	bool		m_bUpdatingControls	= false;
@@ -680,4 +687,7 @@ private:
 	QPoint		m_rulerA;
 	QPoint		m_rulerB;
 	int			m_iTutorialNumber	= -1;		// Tutorial file number 0,1,2,... (or -1 if not in tutorial mode)
+#ifdef _GRID_DEBUG
+	int			m_iGridDebugMode = DEBUGMODE_OFF;
+#endif
 };
