@@ -25,9 +25,10 @@
 #include "finddialog.h"
 #include "pindialog.h"
 #include <QtGlobal>
+#include <QCloseEvent>
 
-static const bool ALLOW_DELAY_BASED_SMART_PAN = true;
-static const bool ALLOW_DELAY_BASED_PAD_SHIFT = true;
+Q_DECL_CONSTEXPR static const bool ALLOW_DELAY_BASED_SMART_PAN = true;
+Q_DECL_CONSTEXPR static const bool ALLOW_DELAY_BASED_PAD_SHIFT = true;
 
 // Following 2 are to slow down the auto-panning while moving components with the mouse
 static std::chrono::steady_clock::time_point g_lastAutoPanTime;
@@ -53,8 +54,10 @@ void MainWindow::GetPixMapXY(const QPoint& currentPoint, int& pixmapX, int& pixm
 	int gndL, gndR, gndT, gndB;
 	m_board.GetGroundFillBounds(gndL, gndR, gndT, gndB);
 
-	pixmapX += gndL;
-	pixmapY += gndT;
+	const int iEdge = static_cast<int>( m_board.GetEdgeWidth() );
+
+	pixmapX += gndL - iEdge;
+	pixmapY += gndT - iEdge;
 }
 
 bool MainWindow::GetRowCol(const QPoint& currentPoint, int& row, int& col, double& deltaRow, double& deltaCol) const
@@ -119,18 +122,33 @@ void MainWindow::wheelEvent(QWheelEvent* event)
 	event->accept();	// If we don't do this, we can get the same event passed multiple times if we're on MS Windows.
 }
 
+void MainWindow::closeEvent(QCloseEvent* event)
+{
+#ifndef VEROROUTE_ANDROID	
+	if ( GetIsModified() )
+	{
+		if ( QMessageBox::question(this, tr("Really Quit?"),
+										 tr("Your layout is not saved. You will lose changes if you quit.  Continue?"),
+										 QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::No )
+			return event->ignore();
+	}
+#endif
+	event->accept();
+}
+
 bool MainWindow::CanModifyRuler() const
 {
 	return m_bRuler && !( m_board.GetCompEdit() || GetSmartPan() || GetShiftKeyDown() || GetResizingText() ||
 						  GetDefiningRect() || GetPaintPins() || GetErasePins() || GetPaintBoard() || GetEraseBoard() || GetPaintFlood() );
 }
 
-bool MainWindow::GetHaveFloatingPin(int& iFloatingNodeId)
+bool MainWindow::GetHaveFloatingPinTH(int& iFloatingNodeId)	// Checks for a floating through-hole pin
 {
 	iFloatingNodeId = BAD_NODEID;
 
 	const Element* pC = m_board.Get(m_board.GetCurrentLayer(), m_gridRow, m_gridCol);
-	if ( pC->GetHasPin() ) return false;	// Skip if we already have a placed pin
+	// This method is called to see if we should automatically erase both layers (to help a floating pin TH be placed)
+	if ( pC->GetHasPin() ) return false;	// Skip if we already have a placed pin on either layer
 
 	for (auto& mapObj : m_board.GetCompMgr().GetMapIdToComp())
 	{
@@ -138,6 +156,7 @@ bool MainWindow::GetHaveFloatingPin(int& iFloatingNodeId)
 		if ( comp.GetIsPlaced() ) continue;	// Only want floating components
 		const auto& eType = comp.GetType();
 		if ( eType == COMP::WIRE || eType == COMP::MARK || eType == COMP::VERO_NUMBER || eType == COMP::VERO_LETTER ) continue;
+		if ( comp.GetIsSOIC() ) continue;	// SOIC parts are not through-hole
 
 		const int j(m_gridRow - comp.GetRow());		if ( j < 0 || j >= comp.GetCompRows() ) continue;
 		const int i(m_gridCol - comp.GetCol());		if ( i < 0 || i >= comp.GetCompCols() ) continue;
@@ -151,7 +170,7 @@ bool MainWindow::GetHaveFloatingPin(int& iFloatingNodeId)
 	return false;
 }
 
-void MainWindow::MousePressEvent(const QPoint& pos, const bool& bLeftClick, const bool& bRightClick)
+void MainWindow::MousePressEvent(const QPoint& pos, bool bLeftClick, bool bRightClick)
 {
 	SetMouseActionString("");
 
@@ -394,7 +413,7 @@ void MainWindow::MousePressEvent(const QPoint& pos, const bool& bLeftClick, cons
 			}
 		}
 
-		const bool bTruePin = pC->GetHasPin() && !pC->GetHasWire();
+		const bool bTruePin = pC->GetLyrHasPin() && !pC->GetHasWire();
 		if ( ( GetPaintPins() || GetErasePins() ) && !bTruePin && !bDoSwap )	// Restrict painting/erasing pins to true pins (not wires)
 			return;
 
@@ -436,9 +455,9 @@ void MainWindow::MousePressEvent(const QPoint& pos, const bool& bLeftClick, cons
 		{
 			// Erase both layers if it makes it easier to let a floating part fall into place.
 			int iFloatingNodeId(BAD_NODEID);
-			const bool bEraseBothLayers = m_board.GetLyrs() > 1 &&				// If 2-layer board ...
-										  !GetPaintPins() && !GetErasePins() &&	// ... and not erasing a pin
-										  GetHaveFloatingPin(iFloatingNodeId);	// ... then see if we have a floating pin and gets its nodeId
+			const bool bEraseBothLayers = m_board.GetLyrs() > 1 &&					// If 2-layer board ...
+										  !GetPaintPins() && !GetErasePins() &&		// ... and not erasing a pin
+										  GetHaveFloatingPinTH(iFloatingNodeId);	// ... then see if we have a floating through-hole pin and gets its nodeId
 
 			bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, BAD_NODEID, GetPaintPins() || GetErasePins());
 			if ( bEraseBothLayers )
@@ -456,10 +475,10 @@ void MainWindow::MousePressEvent(const QPoint& pos, const bool& bLeftClick, cons
 	}
 	else
 	{
-		if ( ALLOW_DELAY_BASED_PAD_SHIFT && pC->GetHasPin() && !pC->GetHasWire() )
+		if ( ALLOW_DELAY_BASED_PAD_SHIFT && pC->GetPinSupportsOffsetPads() )
 			g_bPinClicked = true;
 
-		if ( !pC->GetHasPin() || pC->GetHasWire() )	// Hide the pad offset dialog if we click on a place that cannot have a pad offset
+		if ( !pC->GetPinSupportsOffsetPads() )	// Hide the pad offset dialog if we click on a place that cannot have a pad offset
 			HidePadOffsetDialog();
 	}
 
@@ -514,7 +533,7 @@ void MainWindow::MouseDoubleClickEvent(const QPoint& pos)
 	const int&	layer		= m_board.GetCurrentLayer();
 	Element*	pC			= m_board.Get(layer, m_gridRow, m_gridCol);
 	const bool	bWire		= pC->GetHasWire();
-	const bool	bTruePin	= pC->GetHasPin() && !bWire;
+	const bool	bTruePin	= pC->GetLyrHasPin() && !bWire;
 
 	// Cursor modification
 	centralWidget()->setCursor(Qt::CrossCursor);
@@ -522,12 +541,16 @@ void MainWindow::MouseDoubleClickEvent(const QPoint& pos)
 	const bool bCloseToGridPoint = ( hypot(dRow - 0.5, dCol - 0.5) <= 0.5 );	// true ==> clicked close to grid point
 
 	// Handle changing layer preference for PCBs via double-clicking on a component pin
-	if ( bPCB && bTruePin && !GetPaintAction() && m_board.GetLyrs() == 2 && bCloseToGridPoint )	// Only consider clicks that are close to the grid point
+	if ( bPCB && pC->GetPinSupportsLayerPref() && !GetPaintAction() && m_board.GetLyrs() == 2 && bCloseToGridPoint )	// Only consider clicks that are close to the grid point
 	{
 		const bool bToggled = m_board.ToggleLyrPref(layer, m_gridRow, m_gridCol);	assert(bToggled);
 		if ( bToggled )
 		{
-			SetMouseActionString("change pin layer preference", pC->GetCompId());
+			size_t	pinIndex;
+			int		compId;
+			m_board.GetSlotInfoForTH(pC, pinIndex, compId);
+
+			SetMouseActionString("change pin layer preference", compId);
 
 			// Also handle change of nodeID ...
 			if ( GetCurrentNodeId() != pC->GetNodeId() )
@@ -734,9 +757,9 @@ void MainWindow::MouseMoveEvent(const QPoint& pos)
 		{
 			// Erase both layers if it makes it easier to let a floating part fall into place.
 			int iFloatingNodeId(BAD_NODEID);
-			const bool bEraseBothLayers = m_board.GetLyrs() > 1 && 				// If 2-layer board, and not erasing a pin ...
-										  !GetPaintPins() && !GetErasePins() &&	// ... and not erasing a pin
-										  GetHaveFloatingPin(iFloatingNodeId);	// ... then see if we have a floating pin and gets its nodeId
+			const bool bEraseBothLayers = m_board.GetLyrs() > 1 &&					// If 2-layer board ...
+										  !GetPaintPins() && !GetErasePins() &&		// ... and not erasing a pin
+										  GetHaveFloatingPinTH(iFloatingNodeId);	// ... then see if we have a floating through-hole pin and gets its nodeId
 
 			bool bChanged = m_board.SetNodeIdByUser(layer, m_gridRow, m_gridCol, BAD_NODEID, false);	// false ==> Only allow erase board (not pins)
 			if ( bEraseBothLayers )
@@ -929,6 +952,13 @@ void MainWindow::keyPressEvent(QKeyEvent* event)
 	const int			nComps			= groupMgr.GetNumUserComps();
 	const bool			bIsAutoRepeat	= event->isAutoRepeat();
 
+#ifdef _GRID_DEBUG
+	switch(event->key())
+	{
+		case Qt::Key_F4: m_iGridDebugMode = ( m_iGridDebugMode + 1 ) % DEBUGMODE_END;	break;
+	}
+#endif
+	
 	if ( m_board.GetCompEdit() )
 	{
 		if ( GetCurrentShapeId() != BAD_ID )
